# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .brand import Brand as Brand
from .event import Event as Event
from .meter import Meter as Meter
from .price import Price as Price
from .refund import Refund as Refund
from .dispute import Dispute as Dispute
from .payment import Payment as Payment
from .product import Product as Product
from .currency import Currency as Currency
from .customer import Customer as Customer
from .discount import Discount as Discount
from .conjunction import Conjunction as Conjunction
from .get_dispute import GetDispute as GetDispute
from .license_key import LicenseKey as LicenseKey
from .price_param import PriceParam as PriceParam
from .country_code import CountryCode as CountryCode
from .meter_filter import MeterFilter as MeterFilter
from .subscription import Subscription as Subscription
from .tax_category import TaxCategory as TaxCategory
from .discount_type import DiscountType as DiscountType
from .dispute_stage import DisputeStage as DisputeStage
from .intent_status import IntentStatus as IntentStatus
from .refund_status import RefundStatus as RefundStatus
from .time_interval import TimeInterval as TimeInterval
from .addon_response import AddonResponse as AddonResponse
from .dispute_status import DisputeStatus as DisputeStatus
from .billing_address import BillingAddress as BillingAddress
from .filter_operator import FilterOperator as FilterOperator
from .webhook_details import WebhookDetails as WebhookDetails
from .refund_list_item import RefundListItem as RefundListItem
from .addon_list_params import AddonListParams as AddonListParams
from .event_input_param import EventInputParam as EventInputParam
from .meter_aggregation import MeterAggregation as MeterAggregation
from .meter_list_params import MeterListParams as MeterListParams
from .add_meter_to_price import AddMeterToPrice as AddMeterToPrice
from .attach_addon_param import AttachAddonParam as AttachAddonParam
from .credit_entitlement import CreditEntitlement as CreditEntitlement
from .custom_field_param import CustomFieldParam as CustomFieldParam
from .license_key_status import LicenseKeyStatus as LicenseKeyStatus
from .meter_filter_param import MeterFilterParam as MeterFilterParam
from .new_customer_param import NewCustomerParam as NewCustomerParam
from .payout_list_params import PayoutListParams as PayoutListParams
from .refund_list_params import RefundListParams as RefundListParams
from .theme_config_param import ThemeConfigParam as ThemeConfigParam
from .webhook_event_type import WebhookEventType as WebhookEventType
from .addon_create_params import AddonCreateParams as AddonCreateParams
from .addon_update_params import AddonUpdateParams as AddonUpdateParams
from .brand_create_params import BrandCreateParams as BrandCreateParams
from .brand_list_response import BrandListResponse as BrandListResponse
from .brand_update_params import BrandUpdateParams as BrandUpdateParams
from .dispute_list_params import DisputeListParams as DisputeListParams
from .meter_create_params import MeterCreateParams as MeterCreateParams
from .payment_list_params import PaymentListParams as PaymentListParams
from .product_list_params import ProductListParams as ProductListParams
from .subscription_status import SubscriptionStatus as SubscriptionStatus
from .webhook_list_params import WebhookListParams as WebhookListParams
from .balance_ledger_entry import BalanceLedgerEntry as BalanceLedgerEntry
from .cbb_overage_behavior import CbbOverageBehavior as CbbOverageBehavior
from .customer_list_params import CustomerListParams as CustomerListParams
from .discount_list_params import DiscountListParams as DiscountListParams
from .license_key_duration import LicenseKeyDuration as LicenseKeyDuration
from .license_key_instance import LicenseKeyInstance as LicenseKeyInstance
from .payment_method_types import PaymentMethodTypes as PaymentMethodTypes
from .payout_list_response import PayoutListResponse as PayoutListResponse
from .refund_create_params import RefundCreateParams as RefundCreateParams
from .unwrap_webhook_event import UnwrapWebhookEvent as UnwrapWebhookEvent
from .billing_address_param import BillingAddressParam as BillingAddressParam
from .custom_field_response import CustomFieldResponse as CustomFieldResponse
from .dispute_list_response import DisputeListResponse as DisputeListResponse
from .payment_create_params import PaymentCreateParams as PaymentCreateParams
from .payment_list_response import PaymentListResponse as PaymentListResponse
from .payment_refund_status import PaymentRefundStatus as PaymentRefundStatus
from .product_create_params import ProductCreateParams as ProductCreateParams
from .product_list_response import ProductListResponse as ProductListResponse
from .product_update_params import ProductUpdateParams as ProductUpdateParams
from .webhook_create_params import WebhookCreateParams as WebhookCreateParams
from .webhook_update_params import WebhookUpdateParams as WebhookUpdateParams
from .cbb_proration_behavior import CbbProrationBehavior as CbbProrationBehavior
from .customer_create_params import CustomerCreateParams as CustomerCreateParams
from .customer_request_param import CustomerRequestParam as CustomerRequestParam
from .customer_update_params import CustomerUpdateParams as CustomerUpdateParams
from .discount_create_params import DiscountCreateParams as DiscountCreateParams
from .discount_update_params import DiscountUpdateParams as DiscountUpdateParams
from .product_item_req_param import ProductItemReqParam as ProductItemReqParam
from .checkout_session_status import CheckoutSessionStatus as CheckoutSessionStatus
from .customer_portal_session import CustomerPortalSession as CustomerPortalSession
from .license_activate_params import LicenseActivateParams as LicenseActivateParams
from .license_key_list_params import LicenseKeyListParams as LicenseKeyListParams
from .license_validate_params import LicenseValidateParams as LicenseValidateParams
from .meter_aggregation_param import MeterAggregationParam as MeterAggregationParam
from .payment_create_response import PaymentCreateResponse as PaymentCreateResponse
from .subscription_data_param import SubscriptionDataParam as SubscriptionDataParam
from .theme_mode_config_param import ThemeModeConfigParam as ThemeModeConfigParam
from .usage_event_list_params import UsageEventListParams as UsageEventListParams
from .add_meter_to_price_param import AddMeterToPriceParam as AddMeterToPriceParam
from .addon_cart_response_item import AddonCartResponseItem as AddonCartResponseItem
from .customer_limited_details import CustomerLimitedDetails as CustomerLimitedDetails
from .digital_product_delivery import DigitalProductDelivery as DigitalProductDelivery
from .meter_cart_response_item import MeterCartResponseItem as MeterCartResponseItem
from .subscription_list_params import SubscriptionListParams as SubscriptionListParams
from .checkout_session_response import CheckoutSessionResponse as CheckoutSessionResponse
from .dispute_won_webhook_event import DisputeWonWebhookEvent as DisputeWonWebhookEvent
from .license_activate_response import LicenseActivateResponse as LicenseActivateResponse
from .license_deactivate_params import LicenseDeactivateParams as LicenseDeactivateParams
from .license_key_update_params import LicenseKeyUpdateParams as LicenseKeyUpdateParams
from .license_validate_response import LicenseValidateResponse as LicenseValidateResponse
from .usage_event_ingest_params import UsageEventIngestParams as UsageEventIngestParams
from .credit_added_webhook_event import CreditAddedWebhookEvent as CreditAddedWebhookEvent
from .dispute_lost_webhook_event import DisputeLostWebhookEvent as DisputeLostWebhookEvent
from .license_key_duration_param import LicenseKeyDurationParam as LicenseKeyDurationParam
from .one_time_product_cart_item import OneTimeProductCartItem as OneTimeProductCartItem
from .subscription_charge_params import SubscriptionChargeParams as SubscriptionChargeParams
from .subscription_create_params import SubscriptionCreateParams as SubscriptionCreateParams
from .subscription_list_response import SubscriptionListResponse as SubscriptionListResponse
from .subscription_update_params import SubscriptionUpdateParams as SubscriptionUpdateParams
from .product_update_files_params import ProductUpdateFilesParams as ProductUpdateFilesParams
from .refund_failed_webhook_event import RefundFailedWebhookEvent as RefundFailedWebhookEvent
from .unsafe_unwrap_webhook_event import UnsafeUnwrapWebhookEvent as UnsafeUnwrapWebhookEvent
from .usage_event_ingest_response import UsageEventIngestResponse as UsageEventIngestResponse
from .addon_update_images_response import AddonUpdateImagesResponse as AddonUpdateImagesResponse
from .brand_update_images_response import BrandUpdateImagesResponse as BrandUpdateImagesResponse
from .checkout_session_flags_param import CheckoutSessionFlagsParam as CheckoutSessionFlagsParam
from .credit_expired_webhook_event import CreditExpiredWebhookEvent as CreditExpiredWebhookEvent
from .dispute_opened_webhook_event import DisputeOpenedWebhookEvent as DisputeOpenedWebhookEvent
from .on_demand_subscription_param import OnDemandSubscriptionParam as OnDemandSubscriptionParam
from .payment_failed_webhook_event import PaymentFailedWebhookEvent as PaymentFailedWebhookEvent
from .subscription_charge_response import SubscriptionChargeResponse as SubscriptionChargeResponse
from .subscription_create_response import SubscriptionCreateResponse as SubscriptionCreateResponse
from .credit_deducted_webhook_event import CreditDeductedWebhookEvent as CreditDeductedWebhookEvent
from .digital_product_delivery_file import DigitalProductDeliveryFile as DigitalProductDeliveryFile
from .dispute_expired_webhook_event import DisputeExpiredWebhookEvent as DisputeExpiredWebhookEvent
from .product_update_files_response import ProductUpdateFilesResponse as ProductUpdateFilesResponse
from .attach_existing_customer_param import AttachExistingCustomerParam as AttachExistingCustomerParam
from .balance_retrieve_ledger_params import BalanceRetrieveLedgerParams as BalanceRetrieveLedgerParams
from .checkout_session_create_params import CheckoutSessionCreateParams as CheckoutSessionCreateParams
from .credit_entitlement_list_params import CreditEntitlementListParams as CreditEntitlementListParams
from .dispute_accepted_webhook_event import DisputeAcceptedWebhookEvent as DisputeAcceptedWebhookEvent
from .refund_succeeded_webhook_event import RefundSucceededWebhookEvent as RefundSucceededWebhookEvent
from .attach_credit_entitlement_param import AttachCreditEntitlementParam as AttachCreditEntitlementParam
from .checkout_session_preview_params import CheckoutSessionPreviewParams as CheckoutSessionPreviewParams
from .dispute_cancelled_webhook_event import DisputeCancelledWebhookEvent as DisputeCancelledWebhookEvent
from .payment_cancelled_webhook_event import PaymentCancelledWebhookEvent as PaymentCancelledWebhookEvent
from .payment_succeeded_webhook_event import PaymentSucceededWebhookEvent as PaymentSucceededWebhookEvent
from .subscription_change_plan_params import SubscriptionChangePlanParams as SubscriptionChangePlanParams
from .credit_balance_low_webhook_event import CreditBalanceLowWebhookEvent as CreditBalanceLowWebhookEvent
from .credit_entitlement_cart_response import CreditEntitlementCartResponse as CreditEntitlementCartResponse
from .credit_entitlement_create_params import CreditEntitlementCreateParams as CreditEntitlementCreateParams
from .credit_entitlement_update_params import CreditEntitlementUpdateParams as CreditEntitlementUpdateParams
from .credit_rolled_over_webhook_event import CreditRolledOverWebhookEvent as CreditRolledOverWebhookEvent
from .dispute_challenged_webhook_event import DisputeChallengedWebhookEvent as DisputeChallengedWebhookEvent
from .license_key_instance_list_params import LicenseKeyInstanceListParams as LicenseKeyInstanceListParams
from .payment_processing_webhook_event import PaymentProcessingWebhookEvent as PaymentProcessingWebhookEvent
from .webhook_retrieve_secret_response import WebhookRetrieveSecretResponse as WebhookRetrieveSecretResponse
from .checkout_session_preview_response import CheckoutSessionPreviewResponse as CheckoutSessionPreviewResponse
from .license_key_created_webhook_event import LicenseKeyCreatedWebhookEvent as LicenseKeyCreatedWebhookEvent
from .subscription_active_webhook_event import SubscriptionActiveWebhookEvent as SubscriptionActiveWebhookEvent
from .subscription_failed_webhook_event import SubscriptionFailedWebhookEvent as SubscriptionFailedWebhookEvent
from .license_key_instance_update_params import LicenseKeyInstanceUpdateParams as LicenseKeyInstanceUpdateParams
from .subscription_expired_webhook_event import SubscriptionExpiredWebhookEvent as SubscriptionExpiredWebhookEvent
from .subscription_on_hold_webhook_event import SubscriptionOnHoldWebhookEvent as SubscriptionOnHoldWebhookEvent
from .subscription_renewed_webhook_event import SubscriptionRenewedWebhookEvent as SubscriptionRenewedWebhookEvent
from .subscription_updated_webhook_event import SubscriptionUpdatedWebhookEvent as SubscriptionUpdatedWebhookEvent
from .credit_entitlement_mapping_response import CreditEntitlementMappingResponse as CreditEntitlementMappingResponse
from .checkout_session_customization_param import CheckoutSessionCustomizationParam as CheckoutSessionCustomizationParam
from .credit_overage_charged_webhook_event import CreditOverageChargedWebhookEvent as CreditOverageChargedWebhookEvent
from .payment_retrieve_line_items_response import PaymentRetrieveLineItemsResponse as PaymentRetrieveLineItemsResponse
from .subscription_cancelled_webhook_event import SubscriptionCancelledWebhookEvent as SubscriptionCancelledWebhookEvent
from .checkout_session_billing_address_param import (
    CheckoutSessionBillingAddressParam as CheckoutSessionBillingAddressParam,
)
from .credit_manual_adjustment_webhook_event import (
    CreditManualAdjustmentWebhookEvent as CreditManualAdjustmentWebhookEvent,
)
from .meter_credit_entitlement_cart_response import (
    MeterCreditEntitlementCartResponse as MeterCreditEntitlementCartResponse,
)
from .misc_list_supported_countries_response import (
    MiscListSupportedCountriesResponse as MiscListSupportedCountriesResponse,
)
from .credit_rollover_forfeited_webhook_event import (
    CreditRolloverForfeitedWebhookEvent as CreditRolloverForfeitedWebhookEvent,
)
from .subscription_plan_changed_webhook_event import (
    SubscriptionPlanChangedWebhookEvent as SubscriptionPlanChangedWebhookEvent,
)
from .subscription_preview_change_plan_params import (
    SubscriptionPreviewChangePlanParams as SubscriptionPreviewChangePlanParams,
)
from .subscription_preview_change_plan_response import (
    SubscriptionPreviewChangePlanResponse as SubscriptionPreviewChangePlanResponse,
)
from .subscription_update_payment_method_params import (
    SubscriptionUpdatePaymentMethodParams as SubscriptionUpdatePaymentMethodParams,
)
from .customer_list_credit_entitlements_response import (
    CustomerListCreditEntitlementsResponse as CustomerListCreditEntitlementsResponse,
)
from .customer_retrieve_payment_methods_response import (
    CustomerRetrievePaymentMethodsResponse as CustomerRetrievePaymentMethodsResponse,
)
from .subscription_retrieve_usage_history_params import (
    SubscriptionRetrieveUsageHistoryParams as SubscriptionRetrieveUsageHistoryParams,
)
from .subscription_retrieve_credit_usage_response import (
    SubscriptionRetrieveCreditUsageResponse as SubscriptionRetrieveCreditUsageResponse,
)
from .subscription_update_payment_method_response import (
    SubscriptionUpdatePaymentMethodResponse as SubscriptionUpdatePaymentMethodResponse,
)
from .subscription_retrieve_usage_history_response import (
    SubscriptionRetrieveUsageHistoryResponse as SubscriptionRetrieveUsageHistoryResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from dodopayments import DodoPayments, AsyncDodoPayments
from dodopayments.types import (
    CheckoutSessionStatus,
    CheckoutSessionResponse,
    CheckoutSessionPreviewResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCheckoutSessions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: DodoPayments) -> None:
        checkout_session = client.checkout_sessions.create(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                }
            ],
        )
        assert_matches_type(CheckoutSessionResponse, checkout_session, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: DodoPayments) -> None:
        checkout_session = client.checkout_sessions.create(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                    "addons": [
                        {
                            "addon_id": "addon_id",
                            "quantity": 0,
                        }
                    ],
                    "amount": 0,
                }
            ],
            allowed_payment_method_types=["ach"],
            billing_address={
                "country": "AF",
                "city": "city",
                "state": "state",
                "street": "street",
                "zipcode": "zipcode",
            },
            billing_currency="AED",
            confirm=True,
            custom_fields=[
                {
                    "field_type": "text",
                    "key": "key",
                    "label": "label",
                    "options": ["string"],
                    "placeholder": "placeholder",
                    "required": True,
                }
            ],
            customer={"customer_id": "customer_id"},
            customization={
                "force_language": "force_language",
                "show_on_demand_tag": True,
                "show_order_details": True,
                "theme": "dark",
                "theme_config": {
                    "dark": {
                        "bg_primary": "bg_primary",
                        "bg_secondary": "bg_secondary",
                        "border_primary": "border_primary",
                        "border_secondary": "border_secondary",
                        "button_primary": "button_primary",
                        "button_primary_hover": "button_primary_hover",
                        "button_secondary": "button_secondary",
                        "button_secondary_hover": "button_secondary_hover",
                        "button_text_primary": "button_text_primary",
                        "button_text_secondary": "button_text_secondary",
                        "input_focus_border": "input_focus_border",
                        "text_error": "text_error",
                        "text_placeholder": "text_placeholder",
                        "text_primary": "text_primary",
                        "text_secondary": "text_secondary",
                        "text_success": "text_success",
                    },
                    "font_primary_url": "font_primary_url",
                    "font_secondary_url": "font_secondary_url",
                    "font_size": "xs",
                    "font_weight": "normal",
                    "light": {
                        "bg_primary": "bg_primary",
                        "bg_secondary": "bg_secondary",
                        "border_primary": "border_primary",
                        "border_secondary": "border_secondary",
                        "button_primary": "button_primary",
                        "button_primary_hover": "button_primary_hover",
                        "button_secondary": "button_secondary",
                        "button_secondary_hover": "button_secondary_hover",
                        "button_text_primary": "button_text_primary",
                        "button_text_secondary": "button_text_secondary",
                        "input_focus_border": "input_focus_border",
                        "text_error": "text_error",
                        "text_placeholder": "text_placeholder",
                        "text_primary": "text_primary",
                        "text_secondary": "text_secondary",
                        "text_success": "text_success",
                    },
                    "pay_button_text": "pay_button_text",
                    "radius": "radius",
                },
            },
            discount_code="discount_code",
            feature_flags={
                "allow_currency_selection": True,
                "allow_customer_editing_city": True,
                "allow_customer_editing_country": True,
                "allow_customer_editing_email": True,
                "allow_customer_editing_name": True,
                "allow_customer_editing_state": True,
                "allow_customer_editing_street": True,
                "allow_customer_editing_tax_id": True,
                "allow_customer_editing_zipcode": True,
                "allow_discount_code": True,
                "allow_phone_number_collection": True,
                "allow_tax_id": True,
                "always_create_new_customer": True,
                "redirect_immediately": True,
            },
            force_3ds=True,
            metadata={"foo": "string"},
            minimal_address=True,
            payment_method_id="payment_method_id",
            product_collection_id="product_collection_id",
            return_url="return_url",
            short_link=True,
            show_saved_payment_methods=True,
            subscription_data={
                "on_demand": {
                    "mandate_only": True,
                    "adaptive_currency_fees_inclusive": True,
                    "product_currency": "AED",
                    "product_description": "product_description",
                    "product_price": 0,
                },
                "trial_period_days": 0,
            },
            tax_id="tax_id",
        )
        assert_matches_type(CheckoutSessionResponse, checkout_session, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: DodoPayments) -> None:
        response = client.checkout_sessions.with_raw_response.create(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        checkout_session = response.parse()
        assert_matches_type(CheckoutSessionResponse, checkout_session, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: DodoPayments) -> None:
        with client.checkout_sessions.with_streaming_response.create(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            checkout_session = response.parse()
            assert_matches_type(CheckoutSessionResponse, checkout_session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: DodoPayments) -> None:
        checkout_session = client.checkout_sessions.retrieve(
            "id",
        )
        assert_matches_type(CheckoutSessionStatus, checkout_session, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: DodoPayments) -> None:
        response = client.checkout_sessions.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        checkout_session = response.parse()
        assert_matches_type(CheckoutSessionStatus, checkout_session, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: DodoPayments) -> None:
        with client.checkout_sessions.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            checkout_session = response.parse()
            assert_matches_type(CheckoutSessionStatus, checkout_session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.checkout_sessions.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_preview(self, client: DodoPayments) -> None:
        checkout_session = client.checkout_sessions.preview(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                }
            ],
        )
        assert_matches_type(CheckoutSessionPreviewResponse, checkout_session, path=["response"])

    @parametrize
    def test_method_preview_with_all_params(self, client: DodoPayments) -> None:
        checkout_session = client.checkout_sessions.preview(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                    "addons": [
                        {
                            "addon_id": "addon_id",
                            "quantity": 0,
                        }
                    ],
                    "amount": 0,
                }
            ],
            allowed_payment_method_types=["ach"],
            billing_address={
                "country": "AF",
                "city": "city",
                "state": "state",
                "street": "street",
                "zipcode": "zipcode",
            },
            billing_currency="AED",
            confirm=True,
            custom_fields=[
                {
                    "field_type": "text",
                    "key": "key",
                    "label": "label",
                    "options": ["string"],
                    "placeholder": "placeholder",
                    "required": True,
                }
            ],
            customer={"customer_id": "customer_id"},
            customization={
                "force_language": "force_language",
                "show_on_demand_tag": True,
                "show_order_details": True,
                "theme": "dark",
                "theme_config": {
                    "dark": {
                        "bg_primary": "bg_primary",
                        "bg_secondary": "bg_secondary",
                        "border_primary": "border_primary",
                        "border_secondary": "border_secondary",
                        "button_primary": "button_primary",
                        "button_primary_hover": "button_primary_hover",
                        "button_secondary": "button_secondary",
                        "button_secondary_hover": "button_secondary_hover",
                        "button_text_primary": "button_text_primary",
                        "button_text_secondary": "button_text_secondary",
                        "input_focus_border": "input_focus_border",
                        "text_error": "text_error",
                        "text_placeholder": "text_placeholder",
                        "text_primary": "text_primary",
                        "text_secondary": "text_secondary",
                        "text_success": "text_success",
                    },
                    "font_primary_url": "font_primary_url",
                    "font_secondary_url": "font_secondary_url",
                    "font_size": "xs",
                    "font_weight": "normal",
                    "light": {
                        "bg_primary": "bg_primary",
                        "bg_secondary": "bg_secondary",
                        "border_primary": "border_primary",
                        "border_secondary": "border_secondary",
                        "button_primary": "button_primary",
                        "button_primary_hover": "button_primary_hover",
                        "button_secondary": "button_secondary",
                        "button_secondary_hover": "button_secondary_hover",
                        "button_text_primary": "button_text_primary",
                        "button_text_secondary": "button_text_secondary",
                        "input_focus_border": "input_focus_border",
                        "text_error": "text_error",
                        "text_placeholder": "text_placeholder",
                        "text_primary": "text_primary",
                        "text_secondary": "text_secondary",
                        "text_success": "text_success",
                    },
                    "pay_button_text": "pay_button_text",
                    "radius": "radius",
                },
            },
            discount_code="discount_code",
            feature_flags={
                "allow_currency_selection": True,
                "allow_customer_editing_city": True,
                "allow_customer_editing_country": True,
                "allow_customer_editing_email": True,
                "allow_customer_editing_name": True,
                "allow_customer_editing_state": True,
                "allow_customer_editing_street": True,
                "allow_customer_editing_tax_id": True,
                "allow_customer_editing_zipcode": True,
                "allow_discount_code": True,
                "allow_phone_number_collection": True,
                "allow_tax_id": True,
                "always_create_new_customer": True,
                "redirect_immediately": True,
            },
            force_3ds=True,
            metadata={"foo": "string"},
            minimal_address=True,
            payment_method_id="payment_method_id",
            product_collection_id="product_collection_id",
            return_url="return_url",
            short_link=True,
            show_saved_payment_methods=True,
            subscription_data={
                "on_demand": {
                    "mandate_only": True,
                    "adaptive_currency_fees_inclusive": True,
                    "product_currency": "AED",
                    "product_description": "product_description",
                    "product_price": 0,
                },
                "trial_period_days": 0,
            },
            tax_id="tax_id",
        )
        assert_matches_type(CheckoutSessionPreviewResponse, checkout_session, path=["response"])

    @parametrize
    def test_raw_response_preview(self, client: DodoPayments) -> None:
        response = client.checkout_sessions.with_raw_response.preview(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        checkout_session = response.parse()
        assert_matches_type(CheckoutSessionPreviewResponse, checkout_session, path=["response"])

    @parametrize
    def test_streaming_response_preview(self, client: DodoPayments) -> None:
        with client.checkout_sessions.with_streaming_response.preview(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            checkout_session = response.parse()
            assert_matches_type(CheckoutSessionPreviewResponse, checkout_session, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCheckoutSessions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncDodoPayments) -> None:
        checkout_session = await async_client.checkout_sessions.create(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                }
            ],
        )
        assert_matches_type(CheckoutSessionResponse, checkout_session, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        checkout_session = await async_client.checkout_sessions.create(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                    "addons": [
                        {
                            "addon_id": "addon_id",
                            "quantity": 0,
                        }
                    ],
                    "amount": 0,
                }
            ],
            allowed_payment_method_types=["ach"],
            billing_address={
                "country": "AF",
                "city": "city",
                "state": "state",
                "street": "street",
                "zipcode": "zipcode",
            },
            billing_currency="AED",
            confirm=True,
            custom_fields=[
                {
                    "field_type": "text",
                    "key": "key",
                    "label": "label",
                    "options": ["string"],
                    "placeholder": "placeholder",
                    "required": True,
                }
            ],
            customer={"customer_id": "customer_id"},
            customization={
                "force_language": "force_language",
                "show_on_demand_tag": True,
                "show_order_details": True,
                "theme": "dark",
                "theme_config": {
                    "dark": {
                        "bg_primary": "bg_primary",
                        "bg_secondary": "bg_secondary",
                        "border_primary": "border_primary",
                        "border_secondary": "border_secondary",
                        "button_primary": "button_primary",
                        "button_primary_hover": "button_primary_hover",
                        "button_secondary": "button_secondary",
                        "button_secondary_hover": "button_secondary_hover",
                        "button_text_primary": "button_text_primary",
                        "button_text_secondary": "button_text_secondary",
                        "input_focus_border": "input_focus_border",
                        "text_error": "text_error",
                        "text_placeholder": "text_placeholder",
                        "text_primary": "text_primary",
                        "text_secondary": "text_secondary",
                        "text_success": "text_success",
                    },
                    "font_primary_url": "font_primary_url",
                    "font_secondary_url": "font_secondary_url",
                    "font_size": "xs",
                    "font_weight": "normal",
                    "light": {
                        "bg_primary": "bg_primary",
                        "bg_secondary": "bg_secondary",
                        "border_primary": "border_primary",
                        "border_secondary": "border_secondary",
                        "button_primary": "button_primary",
                        "button_primary_hover": "button_primary_hover",
                        "button_secondary": "button_secondary",
                        "button_secondary_hover": "button_secondary_hover",
                        "button_text_primary": "button_text_primary",
                        "button_text_secondary": "button_text_secondary",
                        "input_focus_border": "input_focus_border",
                        "text_error": "text_error",
                        "text_placeholder": "text_placeholder",
                        "text_primary": "text_primary",
                        "text_secondary": "text_secondary",
                        "text_success": "text_success",
                    },
                    "pay_button_text": "pay_button_text",
                    "radius": "radius",
                },
            },
            discount_code="discount_code",
            feature_flags={
                "allow_currency_selection": True,
                "allow_customer_editing_city": True,
                "allow_customer_editing_country": True,
                "allow_customer_editing_email": True,
                "allow_customer_editing_name": True,
                "allow_customer_editing_state": True,
                "allow_customer_editing_street": True,
                "allow_customer_editing_tax_id": True,
                "allow_customer_editing_zipcode": True,
                "allow_discount_code": True,
                "allow_phone_number_collection": True,
                "allow_tax_id": True,
                "always_create_new_customer": True,
                "redirect_immediately": True,
            },
            force_3ds=True,
            metadata={"foo": "string"},
            minimal_address=True,
            payment_method_id="payment_method_id",
            product_collection_id="product_collection_id",
            return_url="return_url",
            short_link=True,
            show_saved_payment_methods=True,
            subscription_data={
                "on_demand": {
                    "mandate_only": True,
                    "adaptive_currency_fees_inclusive": True,
                    "product_currency": "AED",
                    "product_description": "product_description",
                    "product_price": 0,
                },
                "trial_period_days": 0,
            },
            tax_id="tax_id",
        )
        assert_matches_type(CheckoutSessionResponse, checkout_session, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.checkout_sessions.with_raw_response.create(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        checkout_session = await response.parse()
        assert_matches_type(CheckoutSessionResponse, checkout_session, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.checkout_sessions.with_streaming_response.create(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            checkout_session = await response.parse()
            assert_matches_type(CheckoutSessionResponse, checkout_session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncDodoPayments) -> None:
        checkout_session = await async_client.checkout_sessions.retrieve(
            "id",
        )
        assert_matches_type(CheckoutSessionStatus, checkout_session, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.checkout_sessions.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        checkout_session = await response.parse()
        assert_matches_type(CheckoutSessionStatus, checkout_session, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.checkout_sessions.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            checkout_session = await response.parse()
            assert_matches_type(CheckoutSessionStatus, checkout_session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.checkout_sessions.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_preview(self, async_client: AsyncDodoPayments) -> None:
        checkout_session = await async_client.checkout_sessions.preview(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                }
            ],
        )
        assert_matches_type(CheckoutSessionPreviewResponse, checkout_session, path=["response"])

    @parametrize
    async def test_method_preview_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        checkout_session = await async_client.checkout_sessions.preview(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                    "addons": [
                        {
                            "addon_id": "addon_id",
                            "quantity": 0,
                        }
                    ],
                    "amount": 0,
                }
            ],
            allowed_payment_method_types=["ach"],
            billing_address={
                "country": "AF",
                "city": "city",
                "state": "state",
                "street": "street",
                "zipcode": "zipcode",
            },
            billing_currency="AED",
            confirm=True,
            custom_fields=[
                {
                    "field_type": "text",
                    "key": "key",
                    "label": "label",
                    "options": ["string"],
                    "placeholder": "placeholder",
                    "required": True,
                }
            ],
            customer={"customer_id": "customer_id"},
            customization={
                "force_language": "force_language",
                "show_on_demand_tag": True,
                "show_order_details": True,
                "theme": "dark",
                "theme_config": {
                    "dark": {
                        "bg_primary": "bg_primary",
                        "bg_secondary": "bg_secondary",
                        "border_primary": "border_primary",
                        "border_secondary": "border_secondary",
                        "button_primary": "button_primary",
                        "button_primary_hover": "button_primary_hover",
                        "button_secondary": "button_secondary",
                        "button_secondary_hover": "button_secondary_hover",
                        "button_text_primary": "button_text_primary",
                        "button_text_secondary": "button_text_secondary",
                        "input_focus_border": "input_focus_border",
                        "text_error": "text_error",
                        "text_placeholder": "text_placeholder",
                        "text_primary": "text_primary",
                        "text_secondary": "text_secondary",
                        "text_success": "text_success",
                    },
                    "font_primary_url": "font_primary_url",
                    "font_secondary_url": "font_secondary_url",
                    "font_size": "xs",
                    "font_weight": "normal",
                    "light": {
                        "bg_primary": "bg_primary",
                        "bg_secondary": "bg_secondary",
                        "border_primary": "border_primary",
                        "border_secondary": "border_secondary",
                        "button_primary": "button_primary",
                        "button_primary_hover": "button_primary_hover",
                        "button_secondary": "button_secondary",
                        "button_secondary_hover": "button_secondary_hover",
                        "button_text_primary": "button_text_primary",
                        "button_text_secondary": "button_text_secondary",
                        "input_focus_border": "input_focus_border",
                        "text_error": "text_error",
                        "text_placeholder": "text_placeholder",
                        "text_primary": "text_primary",
                        "text_secondary": "text_secondary",
                        "text_success": "text_success",
                    },
                    "pay_button_text": "pay_button_text",
                    "radius": "radius",
                },
            },
            discount_code="discount_code",
            feature_flags={
                "allow_currency_selection": True,
                "allow_customer_editing_city": True,
                "allow_customer_editing_country": True,
                "allow_customer_editing_email": True,
                "allow_customer_editing_name": True,
                "allow_customer_editing_state": True,
                "allow_customer_editing_street": True,
                "allow_customer_editing_tax_id": True,
                "allow_customer_editing_zipcode": True,
                "allow_discount_code": True,
                "allow_phone_number_collection": True,
                "allow_tax_id": True,
                "always_create_new_customer": True,
                "redirect_immediately": True,
            },
            force_3ds=True,
            metadata={"foo": "string"},
            minimal_address=True,
            payment_method_id="payment_method_id",
            product_collection_id="product_collection_id",
            return_url="return_url",
            short_link=True,
            show_saved_payment_methods=True,
            subscription_data={
                "on_demand": {
                    "mandate_only": True,
                    "adaptive_currency_fees_inclusive": True,
                    "product_currency": "AED",
                    "product_description": "product_description",
                    "product_price": 0,
                },
                "trial_period_days": 0,
            },
            tax_id="tax_id",
        )
        assert_matches_type(CheckoutSessionPreviewResponse, checkout_session, path=["response"])

    @parametrize
    async def test_raw_response_preview(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.checkout_sessions.with_raw_response.preview(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        checkout_session = await response.parse()
        assert_matches_type(CheckoutSessionPreviewResponse, checkout_session, path=["response"])

    @parametrize
    async def test_streaming_response_preview(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.checkout_sessions.with_streaming_response.preview(
            product_cart=[
                {
                    "product_id": "product_id",
                    "quantity": 0,
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            checkout_session = await response.parse()
            assert_matches_type(CheckoutSessionPreviewResponse, checkout_session, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .currency import Currency
from .country_code import CountryCode
from .tax_category import TaxCategory

__all__ = [
    "CheckoutSessionPreviewResponse",
    "CurrentBreakup",
    "ProductCart",
    "ProductCartCreditEntitlement",
    "ProductCartMeter",
    "ProductCartAddon",
    "RecurringBreakup",
]


class CurrentBreakup(BaseModel):
    """Breakup of the current payment"""

    discount: int
    """Total discount amount"""

    subtotal: int
    """Subtotal before discount (pre-tax original prices)"""

    total_amount: int
    """Total amount to be charged (final amount after all calculations)"""

    tax: Optional[int] = None
    """Total tax amount"""


class ProductCartCreditEntitlement(BaseModel):
    """
    Minimal credit entitlement info shown at checkout — what credits the customer will receive
    """

    credit_entitlement_id: str
    """ID of the credit entitlement"""

    credit_entitlement_name: str
    """Name of the credit entitlement"""

    credit_entitlement_unit: str
    """Unit label (e.g. "API Calls", "Tokens")"""

    credits_amount: str
    """Number of credits granted"""


class ProductCartMeter(BaseModel):
    measurement_unit: str

    name: str

    price_per_unit: str

    description: Optional[str] = None

    free_threshold: Optional[int] = None


class ProductCartAddon(BaseModel):
    addon_id: str

    currency: Currency

    discounted_price: int

    name: str

    og_currency: Currency

    og_price: int

    quantity: int

    tax_category: TaxCategory
    """
    Represents the different categories of taxation applicable to various products
    and services.
    """

    tax_inclusive: bool

    tax_rate: int

    description: Optional[str] = None

    discount_amount: Optional[int] = None

    tax: Optional[int] = None


class ProductCart(BaseModel):
    credit_entitlements: List[ProductCartCreditEntitlement]
    """Credit entitlements that will be granted upon purchase"""

    currency: Currency
    """the currency in which the calculatiosn were made"""

    discounted_price: int
    """discounted price"""

    is_subscription: bool
    """Whether this is a subscription product (affects tax calculation in breakup)"""

    is_usage_based: bool

    meters: List[ProductCartMeter]

    og_currency: Currency
    """the product currency"""

    og_price: int
    """original price of the product"""

    product_id: str
    """unique id of the product"""

    quantity: int
    """Quanitity"""

    tax_category: TaxCategory
    """tax category"""

    tax_inclusive: bool
    """Whether tax is included in the price"""

    tax_rate: int
    """tax rate"""

    addons: Optional[List[ProductCartAddon]] = None

    description: Optional[str] = None

    discount_amount: Optional[int] = None
    """discount percentage"""

    discount_cycle: Optional[int] = None
    """number of cycles the discount will apply"""

    name: Optional[str] = None
    """name of the product"""

    tax: Optional[int] = None
    """total tax"""


class RecurringBreakup(BaseModel):
    """Breakup of recurring payments (None for one-time only)"""

    discount: int
    """Total discount amount"""

    subtotal: int
    """Subtotal before discount (pre-tax original prices)"""

    total_amount: int
    """Total recurring amount including tax"""

    tax: Optional[int] = None
    """Total tax on recurring payments"""


class CheckoutSessionPreviewResponse(BaseModel):
    """Data returned by the calculate checkout session API"""

    billing_country: CountryCode
    """Billing country"""

    currency: Currency
    """Currency in which the calculations were made"""

    current_breakup: CurrentBreakup
    """Breakup of the current payment"""

    product_cart: List[ProductCart]
    """The total product cart"""

    total_price: int
    """Total calculate price of the product cart"""

    recurring_breakup: Optional[RecurringBreakup] = None
    """Breakup of recurring payments (None for one-time only)"""

    tax_id_err_msg: Optional[str] = None
    """Error message if tax ID validation failed"""

    total_tax: Optional[int] = None
    """Total tax"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["CreditEntitlementListParams"]


class CreditEntitlementListParams(TypedDict, total=False):
    deleted: bool
    """List deleted credit entitlements"""

    page_number: int
    """Page number default is 0"""

    page_size: int
    """Page size default is 10 max is 100"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel
from .license_key_status import LicenseKeyStatus

__all__ = ["LicenseKey"]


class LicenseKey(BaseModel):
    id: str
    """The unique identifier of the license key."""

    business_id: str
    """The unique identifier of the business associated with the license key."""

    created_at: datetime
    """The timestamp indicating when the license key was created, in UTC."""

    customer_id: str
    """The unique identifier of the customer associated with the license key."""

    instances_count: int
    """The current number of instances activated for this license key."""

    key: str
    """The license key string."""

    payment_id: str
    """The unique identifier of the payment associated with the license key."""

    product_id: str
    """The unique identifier of the product associated with the license key."""

    status: LicenseKeyStatus
    """The current status of the license key (e.g., active, inactive, expired)."""

    activations_limit: Optional[int] = None
    """The maximum number of activations allowed for this license key."""

    expires_at: Optional[datetime] = None
    """The timestamp indicating when the license key expires, in UTC."""

    subscription_id: Optional[str] = None
    """
    The unique identifier of the subscription associated with the license key, if
    any.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional

import httpx

from .images import (
    ImagesResource,
    AsyncImagesResource,
    ImagesResourceWithRawResponse,
    AsyncImagesResourceWithRawResponse,
    ImagesResourceWithStreamingResponse,
    AsyncImagesResourceWithStreamingResponse,
)
from ...types import (
    TaxCategory,
    product_list_params,
    product_create_params,
    product_update_params,
    product_update_files_params,
)
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .short_links import (
    ShortLinksResource,
    AsyncShortLinksResource,
    ShortLinksResourceWithRawResponse,
    AsyncShortLinksResourceWithRawResponse,
    ShortLinksResourceWithStreamingResponse,
    AsyncShortLinksResourceWithStreamingResponse,
)
from ...pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.product import Product
from ...types.price_param import PriceParam
from ...types.tax_category import TaxCategory
from ...types.product_list_response import ProductListResponse
from ...types.license_key_duration_param import LicenseKeyDurationParam
from ...types.product_update_files_response import ProductUpdateFilesResponse
from ...types.attach_credit_entitlement_param import AttachCreditEntitlementParam

__all__ = ["ProductsResource", "AsyncProductsResource"]


class ProductsResource(SyncAPIResource):
    @cached_property
    def images(self) -> ImagesResource:
        return ImagesResource(self._client)

    @cached_property
    def short_links(self) -> ShortLinksResource:
        return ShortLinksResource(self._client)

    @cached_property
    def with_raw_response(self) -> ProductsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return ProductsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProductsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return ProductsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        price: PriceParam,
        tax_category: TaxCategory,
        addons: Optional[SequenceNotStr[str]] | Omit = omit,
        brand_id: Optional[str] | Omit = omit,
        credit_entitlements: Optional[Iterable[AttachCreditEntitlementParam]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        digital_product_delivery: Optional[product_create_params.DigitalProductDelivery] | Omit = omit,
        license_key_activation_message: Optional[str] | Omit = omit,
        license_key_activations_limit: Optional[int] | Omit = omit,
        license_key_duration: Optional[LicenseKeyDurationParam] | Omit = omit,
        license_key_enabled: Optional[bool] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """
        Args:
          name: Name of the product

          price: Price configuration for the product

          tax_category: Tax category applied to this product

          addons: Addons available for subscription product

          brand_id: Brand id for the product, if not provided will default to primary brand

          credit_entitlements: Optional credit entitlements to attach (max 3)

          description: Optional description of the product

          digital_product_delivery: Choose how you would like you digital product delivered

          license_key_activation_message: Optional message displayed during license key activation

          license_key_activations_limit: The number of times the license key can be activated. Must be 0 or greater

          license_key_duration: Duration configuration for the license key. Set to null if you don't want the
              license key to expire. For subscriptions, the lifetime of the license key is
              tied to the subscription period

          license_key_enabled: When true, generates and sends a license key to your customer. Defaults to false

          metadata: Additional metadata for the product

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/products",
            body=maybe_transform(
                {
                    "name": name,
                    "price": price,
                    "tax_category": tax_category,
                    "addons": addons,
                    "brand_id": brand_id,
                    "credit_entitlements": credit_entitlements,
                    "description": description,
                    "digital_product_delivery": digital_product_delivery,
                    "license_key_activation_message": license_key_activation_message,
                    "license_key_activations_limit": license_key_activations_limit,
                    "license_key_duration": license_key_duration,
                    "license_key_enabled": license_key_enabled,
                    "metadata": metadata,
                },
                product_create_params.ProductCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/products/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    def update(
        self,
        id: str,
        *,
        addons: Optional[SequenceNotStr[str]] | Omit = omit,
        brand_id: Optional[str] | Omit = omit,
        credit_entitlements: Optional[Iterable[AttachCreditEntitlementParam]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        digital_product_delivery: Optional[product_update_params.DigitalProductDelivery] | Omit = omit,
        image_id: Optional[str] | Omit = omit,
        license_key_activation_message: Optional[str] | Omit = omit,
        license_key_activations_limit: Optional[int] | Omit = omit,
        license_key_duration: Optional[LicenseKeyDurationParam] | Omit = omit,
        license_key_enabled: Optional[bool] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        price: Optional[PriceParam] | Omit = omit,
        tax_category: Optional[TaxCategory] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Args:
          addons: Available Addons for subscription products

          credit_entitlements: Credit entitlements to update (replaces all existing when present) Send empty
              array to remove all, omit field to leave unchanged

          description: Description of the product, optional and must be at most 1000 characters.

          digital_product_delivery: Choose how you would like you digital product delivered

          image_id: Product image id after its uploaded to S3

          license_key_activation_message: Message sent to the customer upon license key activation.

              Only applicable if `license_key_enabled` is `true`. This message contains
              instructions for activating the license key.

          license_key_activations_limit: Limit for the number of activations for the license key.

              Only applicable if `license_key_enabled` is `true`. Represents the maximum
              number of times the license key can be activated.

          license_key_duration: Duration of the license key if enabled.

              Only applicable if `license_key_enabled` is `true`. Represents the duration in
              days for which the license key is valid.

          license_key_enabled: Whether the product requires a license key.

              If `true`, additional fields related to license key (duration, activations
              limit, activation message) become applicable.

          metadata: Additional metadata for the product

          name: Name of the product, optional and must be at most 100 characters.

          price: Price details of the product.

          tax_category: Tax category of the product.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._patch(
            f"/products/{id}",
            body=maybe_transform(
                {
                    "addons": addons,
                    "brand_id": brand_id,
                    "credit_entitlements": credit_entitlements,
                    "description": description,
                    "digital_product_delivery": digital_product_delivery,
                    "image_id": image_id,
                    "license_key_activation_message": license_key_activation_message,
                    "license_key_activations_limit": license_key_activations_limit,
                    "license_key_duration": license_key_duration,
                    "license_key_enabled": license_key_enabled,
                    "metadata": metadata,
                    "name": name,
                    "price": price,
                    "tax_category": tax_category,
                },
                product_update_params.ProductUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        *,
        archived: bool | Omit = omit,
        brand_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        recurring: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultPageNumberPagination[ProductListResponse]:
        """
        Args:
          archived: List archived products

          brand_id: filter by Brand id

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          recurring:
              Filter products by pricing type:

              - `true`: Show only recurring pricing products (e.g. subscriptions)
              - `false`: Show only one-time price products
              - `null` or absent: Show both types of products

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/products",
            page=SyncDefaultPageNumberPagination[ProductListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "brand_id": brand_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "recurring": recurring,
                    },
                    product_list_params.ProductListParams,
                ),
            ),
            model=ProductListResponse,
        )

    def archive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/products/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def unarchive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/products/{id}/unarchive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def update_files(
        self,
        id: str,
        *,
        file_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductUpdateFilesResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/products/{id}/files",
            body=maybe_transform({"file_name": file_name}, product_update_files_params.ProductUpdateFilesParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductUpdateFilesResponse,
        )


class AsyncProductsResource(AsyncAPIResource):
    @cached_property
    def images(self) -> AsyncImagesResource:
        return AsyncImagesResource(self._client)

    @cached_property
    def short_links(self) -> AsyncShortLinksResource:
        return AsyncShortLinksResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncProductsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncProductsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProductsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncProductsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        price: PriceParam,
        tax_category: TaxCategory,
        addons: Optional[SequenceNotStr[str]] | Omit = omit,
        brand_id: Optional[str] | Omit = omit,
        credit_entitlements: Optional[Iterable[AttachCreditEntitlementParam]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        digital_product_delivery: Optional[product_create_params.DigitalProductDelivery] | Omit = omit,
        license_key_activation_message: Optional[str] | Omit = omit,
        license_key_activations_limit: Optional[int] | Omit = omit,
        license_key_duration: Optional[LicenseKeyDurationParam] | Omit = omit,
        license_key_enabled: Optional[bool] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """
        Args:
          name: Name of the product

          price: Price configuration for the product

          tax_category: Tax category applied to this product

          addons: Addons available for subscription product

          brand_id: Brand id for the product, if not provided will default to primary brand

          credit_entitlements: Optional credit entitlements to attach (max 3)

          description: Optional description of the product

          digital_product_delivery: Choose how you would like you digital product delivered

          license_key_activation_message: Optional message displayed during license key activation

          license_key_activations_limit: The number of times the license key can be activated. Must be 0 or greater

          license_key_duration: Duration configuration for the license key. Set to null if you don't want the
              license key to expire. For subscriptions, the lifetime of the license key is
              tied to the subscription period

          license_key_enabled: When true, generates and sends a license key to your customer. Defaults to false

          metadata: Additional metadata for the product

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/products",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "price": price,
                    "tax_category": tax_category,
                    "addons": addons,
                    "brand_id": brand_id,
                    "credit_entitlements": credit_entitlements,
                    "description": description,
                    "digital_product_delivery": digital_product_delivery,
                    "license_key_activation_message": license_key_activation_message,
                    "license_key_activations_limit": license_key_activations_limit,
                    "license_key_duration": license_key_duration,
                    "license_key_enabled": license_key_enabled,
                    "metadata": metadata,
                },
                product_create_params.ProductCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/products/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    async def update(
        self,
        id: str,
        *,
        addons: Optional[SequenceNotStr[str]] | Omit = omit,
        brand_id: Optional[str] | Omit = omit,
        credit_entitlements: Optional[Iterable[AttachCreditEntitlementParam]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        digital_product_delivery: Optional[product_update_params.DigitalProductDelivery] | Omit = omit,
        image_id: Optional[str] | Omit = omit,
        license_key_activation_message: Optional[str] | Omit = omit,
        license_key_activations_limit: Optional[int] | Omit = omit,
        license_key_duration: Optional[LicenseKeyDurationParam] | Omit = omit,
        license_key_enabled: Optional[bool] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        price: Optional[PriceParam] | Omit = omit,
        tax_category: Optional[TaxCategory] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Args:
          addons: Available Addons for subscription products

          credit_entitlements: Credit entitlements to update (replaces all existing when present) Send empty
              array to remove all, omit field to leave unchanged

          description: Description of the product, optional and must be at most 1000 characters.

          digital_product_delivery: Choose how you would like you digital product delivered

          image_id: Product image id after its uploaded to S3

          license_key_activation_message: Message sent to the customer upon license key activation.

              Only applicable if `license_key_enabled` is `true`. This message contains
              instructions for activating the license key.

          license_key_activations_limit: Limit for the number of activations for the license key.

              Only applicable if `license_key_enabled` is `true`. Represents the maximum
              number of times the license key can be activated.

          license_key_duration: Duration of the license key if enabled.

              Only applicable if `license_key_enabled` is `true`. Represents the duration in
              days for which the license key is valid.

          license_key_enabled: Whether the product requires a license key.

              If `true`, additional fields related to license key (duration, activations
              limit, activation message) become applicable.

          metadata: Additional metadata for the product

          name: Name of the product, optional and must be at most 100 characters.

          price: Price details of the product.

          tax_category: Tax category of the product.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._patch(
            f"/products/{id}",
            body=await async_maybe_transform(
                {
                    "addons": addons,
                    "brand_id": brand_id,
                    "credit_entitlements": credit_entitlements,
                    "description": description,
                    "digital_product_delivery": digital_product_delivery,
                    "image_id": image_id,
                    "license_key_activation_message": license_key_activation_message,
                    "license_key_activations_limit": license_key_activations_limit,
                    "license_key_duration": license_key_duration,
                    "license_key_enabled": license_key_enabled,
                    "metadata": metadata,
                    "name": name,
                    "price": price,
                    "tax_category": tax_category,
                },
                product_update_params.ProductUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        *,
        archived: bool | Omit = omit,
        brand_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        recurring: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ProductListResponse, AsyncDefaultPageNumberPagination[ProductListResponse]]:
        """
        Args:
          archived: List archived products

          brand_id: filter by Brand id

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          recurring:
              Filter products by pricing type:

              - `true`: Show only recurring pricing products (e.g. subscriptions)
              - `false`: Show only one-time price products
              - `null` or absent: Show both types of products

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/products",
            page=AsyncDefaultPageNumberPagination[ProductListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "brand_id": brand_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "recurring": recurring,
                    },
                    product_list_params.ProductListParams,
                ),
            ),
            model=ProductListResponse,
        )

    async def archive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/products/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def unarchive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/products/{id}/unarchive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def update_files(
        self,
        id: str,
        *,
        file_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductUpdateFilesResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/products/{id}/files",
            body=await async_maybe_transform(
                {"file_name": file_name}, product_update_files_params.ProductUpdateFilesParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductUpdateFilesResponse,
        )


class ProductsResourceWithRawResponse:
    def __init__(self, products: ProductsResource) -> None:
        self._products = products

        self.create = to_raw_response_wrapper(
            products.create,
        )
        self.retrieve = to_raw_response_wrapper(
            products.retrieve,
        )
        self.update = to_raw_response_wrapper(
            products.update,
        )
        self.list = to_raw_response_wrapper(
            products.list,
        )
        self.archive = to_raw_response_wrapper(
            products.archive,
        )
        self.unarchive = to_raw_response_wrapper(
            products.unarchive,
        )
        self.update_files = to_raw_response_wrapper(
            products.update_files,
        )

    @cached_property
    def images(self) -> ImagesResourceWithRawResponse:
        return ImagesResourceWithRawResponse(self._products.images)

    @cached_property
    def short_links(self) -> ShortLinksResourceWithRawResponse:
        return ShortLinksResourceWithRawResponse(self._products.short_links)


class AsyncProductsResourceWithRawResponse:
    def __init__(self, products: AsyncProductsResource) -> None:
        self._products = products

        self.create = async_to_raw_response_wrapper(
            products.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            products.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            products.update,
        )
        self.list = async_to_raw_response_wrapper(
            products.list,
        )
        self.archive = async_to_raw_response_wrapper(
            products.archive,
        )
        self.unarchive = async_to_raw_response_wrapper(
            products.unarchive,
        )
        self.update_files = async_to_raw_response_wrapper(
            products.update_files,
        )

    @cached_property
    def images(self) -> AsyncImagesResourceWithRawResponse:
        return AsyncImagesResourceWithRawResponse(self._products.images)

    @cached_property
    def short_links(self) -> AsyncShortLinksResourceWithRawResponse:
        return AsyncShortLinksResourceWithRawResponse(self._products.short_links)


class ProductsResourceWithStreamingResponse:
    def __init__(self, products: ProductsResource) -> None:
        self._products = products

        self.create = to_streamed_response_wrapper(
            products.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            products.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            products.update,
        )
        self.list = to_streamed_response_wrapper(
            products.list,
        )
        self.archive = to_streamed_response_wrapper(
            products.archive,
        )
        self.unarchive = to_streamed_response_wrapper(
            products.unarchive,
        )
        self.update_files = to_streamed_response_wrapper(
            products.update_files,
        )

    @cached_property
    def images(self) -> ImagesResourceWithStreamingResponse:
        return ImagesResourceWithStreamingResponse(self._products.images)

    @cached_property
    def short_links(self) -> ShortLinksResourceWithStreamingResponse:
        return ShortLinksResourceWithStreamingResponse(self._products.short_links)


class AsyncProductsResourceWithStreamingResponse:
    def __init__(self, products: AsyncProductsResource) -> None:
        self._products = products

        self.create = async_to_streamed_response_wrapper(
            products.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            products.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            products.update,
        )
        self.list = async_to_streamed_response_wrapper(
            products.list,
        )
        self.archive = async_to_streamed_response_wrapper(
            products.archive,
        )
        self.unarchive = async_to_streamed_response_wrapper(
            products.unarchive,
        )
        self.update_files = async_to_streamed_response_wrapper(
            products.update_files,
        )

    @cached_property
    def images(self) -> AsyncImagesResourceWithStreamingResponse:
        return AsyncImagesResourceWithStreamingResponse(self._products.images)

    @cached_property
    def short_links(self) -> AsyncShortLinksResourceWithStreamingResponse:
        return AsyncShortLinksResourceWithStreamingResponse(self._products.short_links)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .currency import Currency

__all__ = ["BalanceLedgerEntry"]


class BalanceLedgerEntry(BaseModel):
    id: str

    amount: int

    business_id: str

    created_at: datetime

    currency: Currency

    event_type: Literal[
        "payment",
        "refund",
        "refund_reversal",
        "dispute",
        "dispute_reversal",
        "tax",
        "tax_reversal",
        "payment_fees",
        "refund_fees",
        "refund_fees_reversal",
        "dispute_fees",
        "payout",
        "payout_fees",
        "payout_reversal",
        "payout_fees_reversal",
        "dodo_credits",
        "adjustment",
        "currency_conversion",
    ]

    is_credit: bool

    usd_equivalent_amount: int

    after_balance: Optional[int] = None

    before_balance: Optional[int] = None

    description: Optional[str] = None

    reference_object_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PayoutListParams"]


class PayoutListParams(TypedDict, total=False):
    created_at_gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Get payouts created after this time (inclusive)"""

    created_at_lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Get payouts created before this time (inclusive)"""

    page_number: int
    """Page number default is 0"""

    page_size: int
    """Page size default is 10 max is 100"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from .dispute import Dispute
from .._models import BaseModel

__all__ = ["DisputeExpiredWebhookEvent"]


class DisputeExpiredWebhookEvent(BaseModel):
    business_id: str
    """The business identifier"""

    data: Dispute

    timestamp: datetime
    """The timestamp of when the event occurred"""

    type: Literal["dispute.expired"]
    """The event type"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["DigitalProductDeliveryFile"]


class DigitalProductDeliveryFile(BaseModel):
    file_id: str

    file_name: str

    url: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel
from .currency import Currency
from .time_interval import TimeInterval
from .add_meter_to_price import AddMeterToPrice

__all__ = ["Price", "OneTimePrice", "RecurringPrice", "UsageBasedPrice"]


class OneTimePrice(BaseModel):
    """One-time price details."""

    currency: Currency
    """The currency in which the payment is made."""

    discount: int
    """Discount applied to the price, represented as a percentage (0 to 100)."""

    price: int
    """
    The payment amount, in the smallest denomination of the currency (e.g., cents
    for USD). For example, to charge $1.00, pass `100`.

    If [`pay_what_you_want`](Self::pay_what_you_want) is set to `true`, this field
    represents the **minimum** amount the customer must pay.
    """

    purchasing_power_parity: bool
    """
    Indicates if purchasing power parity adjustments are applied to the price.
    Purchasing power parity feature is not available as of now.
    """

    type: Literal["one_time_price"]

    pay_what_you_want: Optional[bool] = None
    """
    Indicates whether the customer can pay any amount they choose. If set to `true`,
    the [`price`](Self::price) field is the minimum amount.
    """

    suggested_price: Optional[int] = None
    """A suggested price for the user to pay.

    This value is only considered if [`pay_what_you_want`](Self::pay_what_you_want)
    is `true`. Otherwise, it is ignored.
    """

    tax_inclusive: Optional[bool] = None
    """Indicates if the price is tax inclusive."""


class RecurringPrice(BaseModel):
    """Recurring price details."""

    currency: Currency
    """The currency in which the payment is made."""

    discount: int
    """Discount applied to the price, represented as a percentage (0 to 100)."""

    payment_frequency_count: int
    """
    Number of units for the payment frequency. For example, a value of `1` with a
    `payment_frequency_interval` of `month` represents monthly payments.
    """

    payment_frequency_interval: TimeInterval
    """The time interval for the payment frequency (e.g., day, month, year)."""

    price: int
    """The payment amount.

    Represented in the lowest denomination of the currency (e.g., cents for USD).
    For example, to charge $1.00, pass `100`.
    """

    purchasing_power_parity: bool
    """
    Indicates if purchasing power parity adjustments are applied to the price.
    Purchasing power parity feature is not available as of now
    """

    subscription_period_count: int
    """
    Number of units for the subscription period. For example, a value of `12` with a
    `subscription_period_interval` of `month` represents a one-year subscription.
    """

    subscription_period_interval: TimeInterval
    """The time interval for the subscription period (e.g., day, month, year)."""

    type: Literal["recurring_price"]

    tax_inclusive: Optional[bool] = None
    """Indicates if the price is tax inclusive"""

    trial_period_days: Optional[int] = None
    """Number of days for the trial period. A value of `0` indicates no trial period."""


class UsageBasedPrice(BaseModel):
    """Usage Based price details."""

    currency: Currency
    """The currency in which the payment is made."""

    discount: int
    """Discount applied to the price, represented as a percentage (0 to 100)."""

    fixed_price: int
    """The fixed payment amount.

    Represented in the lowest denomination of the currency (e.g., cents for USD).
    For example, to charge $1.00, pass `100`.
    """

    payment_frequency_count: int
    """
    Number of units for the payment frequency. For example, a value of `1` with a
    `payment_frequency_interval` of `month` represents monthly payments.
    """

    payment_frequency_interval: TimeInterval
    """The time interval for the payment frequency (e.g., day, month, year)."""

    purchasing_power_parity: bool
    """
    Indicates if purchasing power parity adjustments are applied to the price.
    Purchasing power parity feature is not available as of now
    """

    subscription_period_count: int
    """
    Number of units for the subscription period. For example, a value of `12` with a
    `subscription_period_interval` of `month` represents a one-year subscription.
    """

    subscription_period_interval: TimeInterval
    """The time interval for the subscription period (e.g., day, month, year)."""

    type: Literal["usage_based_price"]

    meters: Optional[List[AddMeterToPrice]] = None

    tax_inclusive: Optional[bool] = None
    """Indicates if the price is tax inclusive"""


Price: TypeAlias = Union[OneTimePrice, RecurringPrice, UsageBasedPrice]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import TypeAlias

from .new_customer_param import NewCustomerParam
from .attach_existing_customer_param import AttachExistingCustomerParam

__all__ = ["CustomerRequestParam"]

CustomerRequestParam: TypeAlias = Union[AttachExistingCustomerParam, NewCustomerParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CustomerListParams"]


class CustomerListParams(TypedDict, total=False):
    created_at_gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter customers created on or after this timestamp"""

    created_at_lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter customers created on or before this timestamp"""

    email: str
    """Filter by customer email"""

    name: str
    """Filter by customer name (partial match, case-insensitive)"""

    page_number: int
    """Page number default is 0"""

    page_size: int
    """Page size default is 10 max is 100"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union

from .._models import BaseModel
from .conjunction import Conjunction
from .filter_operator import FilterOperator

__all__ = [
    "MeterFilter",
    "ClausesDirectFilterCondition",
    "ClausesNestedMeterFilter",
    "ClausesNestedMeterFilterClausesLevel1FilterCondition",
    "ClausesNestedMeterFilterClausesLevel1NestedFilter",
    "ClausesNestedMeterFilterClausesLevel1NestedFilterClausesLevel2FilterCondition",
    "ClausesNestedMeterFilterClausesLevel1NestedFilterClausesLevel2NestedFilter",
    "ClausesNestedMeterFilterClausesLevel1NestedFilterClausesLevel2NestedFilterClause",
]


class ClausesDirectFilterCondition(BaseModel):
    """Filter condition with key, operator, and value"""

    key: str
    """Filter key to apply"""

    operator: FilterOperator

    value: Union[str, float, bool]
    """Filter value - can be string, number, or boolean"""


class ClausesNestedMeterFilterClausesLevel1FilterCondition(BaseModel):
    """Filter condition with key, operator, and value"""

    key: str
    """Filter key to apply"""

    operator: FilterOperator

    value: Union[str, float, bool]
    """Filter value - can be string, number, or boolean"""


class ClausesNestedMeterFilterClausesLevel1NestedFilterClausesLevel2FilterCondition(BaseModel):
    """Filter condition with key, operator, and value"""

    key: str
    """Filter key to apply"""

    operator: FilterOperator

    value: Union[str, float, bool]
    """Filter value - can be string, number, or boolean"""


class ClausesNestedMeterFilterClausesLevel1NestedFilterClausesLevel2NestedFilterClause(BaseModel):
    """Filter condition with key, operator, and value"""

    key: str
    """Filter key to apply"""

    operator: FilterOperator

    value: Union[str, float, bool]
    """Filter value - can be string, number, or boolean"""


class ClausesNestedMeterFilterClausesLevel1NestedFilterClausesLevel2NestedFilter(BaseModel):
    """Level 3 nested filter (final nesting level)"""

    clauses: List[ClausesNestedMeterFilterClausesLevel1NestedFilterClausesLevel2NestedFilterClause]
    """Level 3: Filter conditions only (max depth reached)"""

    conjunction: Conjunction


class ClausesNestedMeterFilterClausesLevel1NestedFilter(BaseModel):
    """Level 2 nested filter"""

    clauses: Union[
        List[ClausesNestedMeterFilterClausesLevel1NestedFilterClausesLevel2FilterCondition],
        List[ClausesNestedMeterFilterClausesLevel1NestedFilterClausesLevel2NestedFilter],
    ]
    """Level 2: Can be conditions or nested filters (1 more level allowed)"""

    conjunction: Conjunction


class ClausesNestedMeterFilter(BaseModel):
    """Level 1 nested filter - can contain Level 2 filters"""

    clauses: Union[
        List[ClausesNestedMeterFilterClausesLevel1FilterCondition],
        List[ClausesNestedMeterFilterClausesLevel1NestedFilter],
    ]
    """Level 1: Can be conditions or nested filters (2 more levels allowed)"""

    conjunction: Conjunction


class MeterFilter(BaseModel):
    """
    A filter structure that combines multiple conditions with logical conjunctions (AND/OR).

    Supports up to 3 levels of nesting to create complex filter expressions.
    Each filter has a conjunction (and/or) and clauses that can be either direct conditions or nested filters.
    """

    clauses: Union[List[ClausesDirectFilterCondition], List[ClausesNestedMeterFilter]]
    """
    Filter clauses - can be direct conditions or nested filters (up to 3 levels
    deep)
    """

    conjunction: Conjunction
    """Logical conjunction to apply between clauses (and/or)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ...types import (
    Currency,
    TimeInterval,
    CbbOverageBehavior,
    credit_entitlement_list_params,
    credit_entitlement_create_params,
    credit_entitlement_update_params,
)
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .balances import (
    BalancesResource,
    AsyncBalancesResource,
    BalancesResourceWithRawResponse,
    AsyncBalancesResourceWithRawResponse,
    BalancesResourceWithStreamingResponse,
    AsyncBalancesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.currency import Currency
from ...types.time_interval import TimeInterval
from ...types.credit_entitlement import CreditEntitlement
from ...types.cbb_overage_behavior import CbbOverageBehavior

__all__ = ["CreditEntitlementsResource", "AsyncCreditEntitlementsResource"]


class CreditEntitlementsResource(SyncAPIResource):
    @cached_property
    def balances(self) -> BalancesResource:
        return BalancesResource(self._client)

    @cached_property
    def with_raw_response(self) -> CreditEntitlementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return CreditEntitlementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CreditEntitlementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return CreditEntitlementsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        overage_enabled: bool,
        precision: int,
        rollover_enabled: bool,
        unit: str,
        currency: Optional[Currency] | Omit = omit,
        description: Optional[str] | Omit = omit,
        expires_after_days: Optional[int] | Omit = omit,
        max_rollover_count: Optional[int] | Omit = omit,
        overage_behavior: Optional[CbbOverageBehavior] | Omit = omit,
        overage_limit: Optional[int] | Omit = omit,
        price_per_unit: Optional[str] | Omit = omit,
        rollover_percentage: Optional[int] | Omit = omit,
        rollover_timeframe_count: Optional[int] | Omit = omit,
        rollover_timeframe_interval: Optional[TimeInterval] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditEntitlement:
        """
        Credit entitlements define reusable credit templates that can be attached to
        products. Each entitlement defines how credits behave in terms of expiration,
        rollover, and overage.

        # Authentication

        Requires an API key with `Editor` role.

        # Request Body

        - `name` - Human-readable name of the credit entitlement (1-255 characters,
          required)
        - `description` - Optional description (max 1000 characters)
        - `precision` - Decimal precision for credit amounts (0-10 decimal places)
        - `unit` - Unit of measurement for the credit (e.g., "API Calls", "Tokens",
          "Credits")
        - `expires_after_days` - Number of days after which credits expire (optional)
        - `rollover_enabled` - Whether unused credits can rollover to the next period
        - `rollover_percentage` - Percentage of unused credits that rollover (0-100)
        - `rollover_timeframe_count` - Count of timeframe periods for rollover limit
        - `rollover_timeframe_interval` - Interval type (day, week, month, year)
        - `max_rollover_count` - Maximum number of times credits can be rolled over
        - `overage_enabled` - Whether overage charges apply when credits run out
          (requires price_per_unit)
        - `overage_limit` - Maximum overage units allowed (optional)
        - `currency` - Currency for pricing (required if price_per_unit is set)
        - `price_per_unit` - Price per credit unit (decimal)

        # Responses

        - `201 Created` - Credit entitlement created successfully, returns the full
          entitlement object
        - `422 Unprocessable Entity` - Invalid request parameters or validation failure
        - `500 Internal Server Error` - Database or server error

        # Business Logic

        - A unique ID with prefix `cde_` is automatically generated for the entitlement
        - Created and updated timestamps are automatically set
        - Currency is required when price_per_unit is set
        - price_per_unit is required when overage_enabled is true
        - rollover_timeframe_count and rollover_timeframe_interval must both be set or
          both be null

        Args:
          name: Name of the credit entitlement

          overage_enabled: Whether overage charges are enabled when credits run out

          precision: Precision for credit amounts (0-10 decimal places)

          rollover_enabled: Whether rollover is enabled for unused credits

          unit: Unit of measurement for the credit (e.g., "API Calls", "Tokens", "Credits")

          currency: Currency for pricing (required if price_per_unit is set)

          description: Optional description of the credit entitlement

          expires_after_days: Number of days after which credits expire (optional)

          max_rollover_count: Maximum number of times credits can be rolled over

          overage_behavior: Controls how overage is handled at billing cycle end. Defaults to
              forgive_at_reset if not specified.

          overage_limit: Maximum overage units allowed (optional)

          price_per_unit: Price per credit unit

          rollover_percentage: Percentage of unused credits that can rollover (0-100)

          rollover_timeframe_count: Count of timeframe periods for rollover limit

          rollover_timeframe_interval: Interval type for rollover timeframe

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/credit-entitlements",
            body=maybe_transform(
                {
                    "name": name,
                    "overage_enabled": overage_enabled,
                    "precision": precision,
                    "rollover_enabled": rollover_enabled,
                    "unit": unit,
                    "currency": currency,
                    "description": description,
                    "expires_after_days": expires_after_days,
                    "max_rollover_count": max_rollover_count,
                    "overage_behavior": overage_behavior,
                    "overage_limit": overage_limit,
                    "price_per_unit": price_per_unit,
                    "rollover_percentage": rollover_percentage,
                    "rollover_timeframe_count": rollover_timeframe_count,
                    "rollover_timeframe_interval": rollover_timeframe_interval,
                },
                credit_entitlement_create_params.CreditEntitlementCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditEntitlement,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditEntitlement:
        """
        Returns the full details of a single credit entitlement including all
        configuration settings for expiration, rollover, and overage policies.

        # Authentication

        Requires an API key with `Viewer` role or higher.

        # Path Parameters

        - `id` - The unique identifier of the credit entitlement (format: `cde_...`)

        # Responses

        - `200 OK` - Returns the full credit entitlement object
        - `404 Not Found` - Credit entitlement does not exist or does not belong to the
          authenticated business
        - `500 Internal Server Error` - Database or server error

        # Business Logic

        - Only non-deleted credit entitlements can be retrieved through this endpoint
        - The entitlement must belong to the authenticated business (business_id check)
        - Deleted entitlements return a 404 error and must be retrieved via the list
          endpoint with `deleted=true`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/credit-entitlements/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditEntitlement,
        )

    def update(
        self,
        id: str,
        *,
        currency: Optional[Currency] | Omit = omit,
        description: Optional[str] | Omit = omit,
        expires_after_days: Optional[int] | Omit = omit,
        max_rollover_count: Optional[int] | Omit = omit,
        name: Optional[str] | Omit = omit,
        overage_behavior: Optional[CbbOverageBehavior] | Omit = omit,
        overage_enabled: Optional[bool] | Omit = omit,
        overage_limit: Optional[int] | Omit = omit,
        price_per_unit: Optional[str] | Omit = omit,
        rollover_enabled: Optional[bool] | Omit = omit,
        rollover_percentage: Optional[int] | Omit = omit,
        rollover_timeframe_count: Optional[int] | Omit = omit,
        rollover_timeframe_interval: Optional[TimeInterval] | Omit = omit,
        unit: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Allows partial updates to a credit entitlement's configuration.

        Only the fields
        provided in the request body will be updated; all other fields remain unchanged.
        This endpoint supports nullable fields using the double option pattern.

        # Authentication

        Requires an API key with `Editor` role.

        # Path Parameters

        - `id` - The unique identifier of the credit entitlement to update (format:
          `cde_...`)

        # Request Body (all fields optional)

        - `name` - Human-readable name of the credit entitlement (1-255 characters)
        - `description` - Optional description (max 1000 characters)
        - `unit` - Unit of measurement for the credit (1-50 characters)

        Note: `precision` cannot be modified after creation as it would invalidate
        existing grants.

        - `expires_after_days` - Number of days after which credits expire (use `null`
          to remove expiration)
        - `rollover_enabled` - Whether unused credits can rollover to the next period
        - `rollover_percentage` - Percentage of unused credits that rollover (0-100,
          nullable)
        - `rollover_timeframe_count` - Count of timeframe periods for rollover limit
          (nullable)
        - `rollover_timeframe_interval` - Interval type (day, week, month, year,
          nullable)
        - `max_rollover_count` - Maximum number of times credits can be rolled over
          (nullable)
        - `overage_enabled` - Whether overage charges apply when credits run out
        - `overage_limit` - Maximum overage units allowed (nullable)
        - `currency` - Currency for pricing (nullable)
        - `price_per_unit` - Price per credit unit (decimal, nullable)

        # Responses

        - `200 OK` - Credit entitlement updated successfully
        - `404 Not Found` - Credit entitlement does not exist or does not belong to the
          authenticated business
        - `422 Unprocessable Entity` - Invalid request parameters or validation failure
        - `500 Internal Server Error` - Database or server error

        # Business Logic

        - Only non-deleted credit entitlements can be updated
        - Fields set to `null` explicitly will clear the database value (using double
          option pattern)
        - The `updated_at` timestamp is automatically updated on successful modification
        - Changes take effect immediately but do not retroactively affect existing
          credit grants
        - The merged state is validated: currency required with price, rollover
          timeframe fields together, price required for overage

        Args:
          currency: Currency for pricing

          description: Optional description of the credit entitlement

          expires_after_days: Number of days after which credits expire

          max_rollover_count: Maximum number of times credits can be rolled over

          name: Name of the credit entitlement

          overage_behavior: Controls how overage is handled at billing cycle end.

          overage_enabled: Whether overage charges are enabled when credits run out

          overage_limit: Maximum overage units allowed

          price_per_unit: Price per credit unit

          rollover_enabled: Whether rollover is enabled for unused credits

          rollover_percentage: Percentage of unused credits that can rollover (0-100)

          rollover_timeframe_count: Count of timeframe periods for rollover limit

          rollover_timeframe_interval: Interval type for rollover timeframe

          unit: Unit of measurement for the credit (e.g., "API Calls", "Tokens", "Credits")

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._patch(
            f"/credit-entitlements/{id}",
            body=maybe_transform(
                {
                    "currency": currency,
                    "description": description,
                    "expires_after_days": expires_after_days,
                    "max_rollover_count": max_rollover_count,
                    "name": name,
                    "overage_behavior": overage_behavior,
                    "overage_enabled": overage_enabled,
                    "overage_limit": overage_limit,
                    "price_per_unit": price_per_unit,
                    "rollover_enabled": rollover_enabled,
                    "rollover_percentage": rollover_percentage,
                    "rollover_timeframe_count": rollover_timeframe_count,
                    "rollover_timeframe_interval": rollover_timeframe_interval,
                    "unit": unit,
                },
                credit_entitlement_update_params.CreditEntitlementUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        *,
        deleted: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultPageNumberPagination[CreditEntitlement]:
        """
        Returns a paginated list of credit entitlements, allowing filtering of deleted
        entitlements. By default, only non-deleted entitlements are returned.

        # Authentication

        Requires an API key with `Viewer` role or higher.

        # Query Parameters

        - `page_size` - Number of items per page (default: 10, max: 100)
        - `page_number` - Zero-based page number (default: 0)
        - `deleted` - Boolean flag to list deleted entitlements instead of active ones
          (default: false)

        # Responses

        - `200 OK` - Returns a list of credit entitlements wrapped in a response object
        - `422 Unprocessable Entity` - Invalid query parameters (e.g., page_size > 100)
        - `500 Internal Server Error` - Database or server error

        # Business Logic

        - Results are ordered by creation date in descending order (newest first)
        - Only entitlements belonging to the authenticated business are returned
        - The `deleted` parameter controls visibility of soft-deleted entitlements
        - Pagination uses offset-based pagination (offset = page_number \\** page_size)

        Args:
          deleted: List deleted credit entitlements

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/credit-entitlements",
            page=SyncDefaultPageNumberPagination[CreditEntitlement],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "deleted": deleted,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    credit_entitlement_list_params.CreditEntitlementListParams,
                ),
            ),
            model=CreditEntitlement,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/credit-entitlements/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def undelete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Undeletes a soft-deleted credit entitlement by clearing `deleted_at`, making it
        available again through standard list and get endpoints.

        # Authentication

        Requires an API key with `Editor` role.

        # Path Parameters

        - `id` - The unique identifier of the credit entitlement to restore (format:
          `cde_...`)

        # Responses

        - `200 OK` - Credit entitlement restored successfully
        - `500 Internal Server Error` - Database error, entitlement not found, or
          entitlement is not deleted

        # Business Logic

        - Only deleted credit entitlements can be restored
        - The query filters for `deleted_at IS NOT NULL`, so non-deleted entitlements
          will result in 0 rows affected
        - If no rows are affected (entitlement doesn't exist, doesn't belong to
          business, or is not deleted), returns 500
        - The `updated_at` timestamp is automatically updated on successful restoration
        - Once restored, the entitlement becomes immediately available in the standard
          list and get endpoints
        - All configuration settings are preserved during delete/restore operations

        # Error Handling

        This endpoint returns 500 Internal Server Error in several cases:

        - The credit entitlement does not exist
        - The credit entitlement belongs to a different business
        - The credit entitlement is not currently deleted (already active)

        Callers should verify the entitlement exists and is deleted before calling this
        endpoint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/credit-entitlements/{id}/undelete",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncCreditEntitlementsResource(AsyncAPIResource):
    @cached_property
    def balances(self) -> AsyncBalancesResource:
        return AsyncBalancesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCreditEntitlementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCreditEntitlementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCreditEntitlementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncCreditEntitlementsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        overage_enabled: bool,
        precision: int,
        rollover_enabled: bool,
        unit: str,
        currency: Optional[Currency] | Omit = omit,
        description: Optional[str] | Omit = omit,
        expires_after_days: Optional[int] | Omit = omit,
        max_rollover_count: Optional[int] | Omit = omit,
        overage_behavior: Optional[CbbOverageBehavior] | Omit = omit,
        overage_limit: Optional[int] | Omit = omit,
        price_per_unit: Optional[str] | Omit = omit,
        rollover_percentage: Optional[int] | Omit = omit,
        rollover_timeframe_count: Optional[int] | Omit = omit,
        rollover_timeframe_interval: Optional[TimeInterval] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditEntitlement:
        """
        Credit entitlements define reusable credit templates that can be attached to
        products. Each entitlement defines how credits behave in terms of expiration,
        rollover, and overage.

        # Authentication

        Requires an API key with `Editor` role.

        # Request Body

        - `name` - Human-readable name of the credit entitlement (1-255 characters,
          required)
        - `description` - Optional description (max 1000 characters)
        - `precision` - Decimal precision for credit amounts (0-10 decimal places)
        - `unit` - Unit of measurement for the credit (e.g., "API Calls", "Tokens",
          "Credits")
        - `expires_after_days` - Number of days after which credits expire (optional)
        - `rollover_enabled` - Whether unused credits can rollover to the next period
        - `rollover_percentage` - Percentage of unused credits that rollover (0-100)
        - `rollover_timeframe_count` - Count of timeframe periods for rollover limit
        - `rollover_timeframe_interval` - Interval type (day, week, month, year)
        - `max_rollover_count` - Maximum number of times credits can be rolled over
        - `overage_enabled` - Whether overage charges apply when credits run out
          (requires price_per_unit)
        - `overage_limit` - Maximum overage units allowed (optional)
        - `currency` - Currency for pricing (required if price_per_unit is set)
        - `price_per_unit` - Price per credit unit (decimal)

        # Responses

        - `201 Created` - Credit entitlement created successfully, returns the full
          entitlement object
        - `422 Unprocessable Entity` - Invalid request parameters or validation failure
        - `500 Internal Server Error` - Database or server error

        # Business Logic

        - A unique ID with prefix `cde_` is automatically generated for the entitlement
        - Created and updated timestamps are automatically set
        - Currency is required when price_per_unit is set
        - price_per_unit is required when overage_enabled is true
        - rollover_timeframe_count and rollover_timeframe_interval must both be set or
          both be null

        Args:
          name: Name of the credit entitlement

          overage_enabled: Whether overage charges are enabled when credits run out

          precision: Precision for credit amounts (0-10 decimal places)

          rollover_enabled: Whether rollover is enabled for unused credits

          unit: Unit of measurement for the credit (e.g., "API Calls", "Tokens", "Credits")

          currency: Currency for pricing (required if price_per_unit is set)

          description: Optional description of the credit entitlement

          expires_after_days: Number of days after which credits expire (optional)

          max_rollover_count: Maximum number of times credits can be rolled over

          overage_behavior: Controls how overage is handled at billing cycle end. Defaults to
              forgive_at_reset if not specified.

          overage_limit: Maximum overage units allowed (optional)

          price_per_unit: Price per credit unit

          rollover_percentage: Percentage of unused credits that can rollover (0-100)

          rollover_timeframe_count: Count of timeframe periods for rollover limit

          rollover_timeframe_interval: Interval type for rollover timeframe

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/credit-entitlements",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "overage_enabled": overage_enabled,
                    "precision": precision,
                    "rollover_enabled": rollover_enabled,
                    "unit": unit,
                    "currency": currency,
                    "description": description,
                    "expires_after_days": expires_after_days,
                    "max_rollover_count": max_rollover_count,
                    "overage_behavior": overage_behavior,
                    "overage_limit": overage_limit,
                    "price_per_unit": price_per_unit,
                    "rollover_percentage": rollover_percentage,
                    "rollover_timeframe_count": rollover_timeframe_count,
                    "rollover_timeframe_interval": rollover_timeframe_interval,
                },
                credit_entitlement_create_params.CreditEntitlementCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditEntitlement,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditEntitlement:
        """
        Returns the full details of a single credit entitlement including all
        configuration settings for expiration, rollover, and overage policies.

        # Authentication

        Requires an API key with `Viewer` role or higher.

        # Path Parameters

        - `id` - The unique identifier of the credit entitlement (format: `cde_...`)

        # Responses

        - `200 OK` - Returns the full credit entitlement object
        - `404 Not Found` - Credit entitlement does not exist or does not belong to the
          authenticated business
        - `500 Internal Server Error` - Database or server error

        # Business Logic

        - Only non-deleted credit entitlements can be retrieved through this endpoint
        - The entitlement must belong to the authenticated business (business_id check)
        - Deleted entitlements return a 404 error and must be retrieved via the list
          endpoint with `deleted=true`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/credit-entitlements/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditEntitlement,
        )

    async def update(
        self,
        id: str,
        *,
        currency: Optional[Currency] | Omit = omit,
        description: Optional[str] | Omit = omit,
        expires_after_days: Optional[int] | Omit = omit,
        max_rollover_count: Optional[int] | Omit = omit,
        name: Optional[str] | Omit = omit,
        overage_behavior: Optional[CbbOverageBehavior] | Omit = omit,
        overage_enabled: Optional[bool] | Omit = omit,
        overage_limit: Optional[int] | Omit = omit,
        price_per_unit: Optional[str] | Omit = omit,
        rollover_enabled: Optional[bool] | Omit = omit,
        rollover_percentage: Optional[int] | Omit = omit,
        rollover_timeframe_count: Optional[int] | Omit = omit,
        rollover_timeframe_interval: Optional[TimeInterval] | Omit = omit,
        unit: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Allows partial updates to a credit entitlement's configuration.

        Only the fields
        provided in the request body will be updated; all other fields remain unchanged.
        This endpoint supports nullable fields using the double option pattern.

        # Authentication

        Requires an API key with `Editor` role.

        # Path Parameters

        - `id` - The unique identifier of the credit entitlement to update (format:
          `cde_...`)

        # Request Body (all fields optional)

        - `name` - Human-readable name of the credit entitlement (1-255 characters)
        - `description` - Optional description (max 1000 characters)
        - `unit` - Unit of measurement for the credit (1-50 characters)

        Note: `precision` cannot be modified after creation as it would invalidate
        existing grants.

        - `expires_after_days` - Number of days after which credits expire (use `null`
          to remove expiration)
        - `rollover_enabled` - Whether unused credits can rollover to the next period
        - `rollover_percentage` - Percentage of unused credits that rollover (0-100,
          nullable)
        - `rollover_timeframe_count` - Count of timeframe periods for rollover limit
          (nullable)
        - `rollover_timeframe_interval` - Interval type (day, week, month, year,
          nullable)
        - `max_rollover_count` - Maximum number of times credits can be rolled over
          (nullable)
        - `overage_enabled` - Whether overage charges apply when credits run out
        - `overage_limit` - Maximum overage units allowed (nullable)
        - `currency` - Currency for pricing (nullable)
        - `price_per_unit` - Price per credit unit (decimal, nullable)

        # Responses

        - `200 OK` - Credit entitlement updated successfully
        - `404 Not Found` - Credit entitlement does not exist or does not belong to the
          authenticated business
        - `422 Unprocessable Entity` - Invalid request parameters or validation failure
        - `500 Internal Server Error` - Database or server error

        # Business Logic

        - Only non-deleted credit entitlements can be updated
        - Fields set to `null` explicitly will clear the database value (using double
          option pattern)
        - The `updated_at` timestamp is automatically updated on successful modification
        - Changes take effect immediately but do not retroactively affect existing
          credit grants
        - The merged state is validated: currency required with price, rollover
          timeframe fields together, price required for overage

        Args:
          currency: Currency for pricing

          description: Optional description of the credit entitlement

          expires_after_days: Number of days after which credits expire

          max_rollover_count: Maximum number of times credits can be rolled over

          name: Name of the credit entitlement

          overage_behavior: Controls how overage is handled at billing cycle end.

          overage_enabled: Whether overage charges are enabled when credits run out

          overage_limit: Maximum overage units allowed

          price_per_unit: Price per credit unit

          rollover_enabled: Whether rollover is enabled for unused credits

          rollover_percentage: Percentage of unused credits that can rollover (0-100)

          rollover_timeframe_count: Count of timeframe periods for rollover limit

          rollover_timeframe_interval: Interval type for rollover timeframe

          unit: Unit of measurement for the credit (e.g., "API Calls", "Tokens", "Credits")

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._patch(
            f"/credit-entitlements/{id}",
            body=await async_maybe_transform(
                {
                    "currency": currency,
                    "description": description,
                    "expires_after_days": expires_after_days,
                    "max_rollover_count": max_rollover_count,
                    "name": name,
                    "overage_behavior": overage_behavior,
                    "overage_enabled": overage_enabled,
                    "overage_limit": overage_limit,
                    "price_per_unit": price_per_unit,
                    "rollover_enabled": rollover_enabled,
                    "rollover_percentage": rollover_percentage,
                    "rollover_timeframe_count": rollover_timeframe_count,
                    "rollover_timeframe_interval": rollover_timeframe_interval,
                    "unit": unit,
                },
                credit_entitlement_update_params.CreditEntitlementUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        *,
        deleted: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CreditEntitlement, AsyncDefaultPageNumberPagination[CreditEntitlement]]:
        """
        Returns a paginated list of credit entitlements, allowing filtering of deleted
        entitlements. By default, only non-deleted entitlements are returned.

        # Authentication

        Requires an API key with `Viewer` role or higher.

        # Query Parameters

        - `page_size` - Number of items per page (default: 10, max: 100)
        - `page_number` - Zero-based page number (default: 0)
        - `deleted` - Boolean flag to list deleted entitlements instead of active ones
          (default: false)

        # Responses

        - `200 OK` - Returns a list of credit entitlements wrapped in a response object
        - `422 Unprocessable Entity` - Invalid query parameters (e.g., page_size > 100)
        - `500 Internal Server Error` - Database or server error

        # Business Logic

        - Results are ordered by creation date in descending order (newest first)
        - Only entitlements belonging to the authenticated business are returned
        - The `deleted` parameter controls visibility of soft-deleted entitlements
        - Pagination uses offset-based pagination (offset = page_number \\** page_size)

        Args:
          deleted: List deleted credit entitlements

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/credit-entitlements",
            page=AsyncDefaultPageNumberPagination[CreditEntitlement],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "deleted": deleted,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    credit_entitlement_list_params.CreditEntitlementListParams,
                ),
            ),
            model=CreditEntitlement,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/credit-entitlements/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def undelete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Undeletes a soft-deleted credit entitlement by clearing `deleted_at`, making it
        available again through standard list and get endpoints.

        # Authentication

        Requires an API key with `Editor` role.

        # Path Parameters

        - `id` - The unique identifier of the credit entitlement to restore (format:
          `cde_...`)

        # Responses

        - `200 OK` - Credit entitlement restored successfully
        - `500 Internal Server Error` - Database error, entitlement not found, or
          entitlement is not deleted

        # Business Logic

        - Only deleted credit entitlements can be restored
        - The query filters for `deleted_at IS NOT NULL`, so non-deleted entitlements
          will result in 0 rows affected
        - If no rows are affected (entitlement doesn't exist, doesn't belong to
          business, or is not deleted), returns 500
        - The `updated_at` timestamp is automatically updated on successful restoration
        - Once restored, the entitlement becomes immediately available in the standard
          list and get endpoints
        - All configuration settings are preserved during delete/restore operations

        # Error Handling

        This endpoint returns 500 Internal Server Error in several cases:

        - The credit entitlement does not exist
        - The credit entitlement belongs to a different business
        - The credit entitlement is not currently deleted (already active)

        Callers should verify the entitlement exists and is deleted before calling this
        endpoint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/credit-entitlements/{id}/undelete",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class CreditEntitlementsResourceWithRawResponse:
    def __init__(self, credit_entitlements: CreditEntitlementsResource) -> None:
        self._credit_entitlements = credit_entitlements

        self.create = to_raw_response_wrapper(
            credit_entitlements.create,
        )
        self.retrieve = to_raw_response_wrapper(
            credit_entitlements.retrieve,
        )
        self.update = to_raw_response_wrapper(
            credit_entitlements.update,
        )
        self.list = to_raw_response_wrapper(
            credit_entitlements.list,
        )
        self.delete = to_raw_response_wrapper(
            credit_entitlements.delete,
        )
        self.undelete = to_raw_response_wrapper(
            credit_entitlements.undelete,
        )

    @cached_property
    def balances(self) -> BalancesResourceWithRawResponse:
        return BalancesResourceWithRawResponse(self._credit_entitlements.balances)


class AsyncCreditEntitlementsResourceWithRawResponse:
    def __init__(self, credit_entitlements: AsyncCreditEntitlementsResource) -> None:
        self._credit_entitlements = credit_entitlements

        self.create = async_to_raw_response_wrapper(
            credit_entitlements.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            credit_entitlements.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            credit_entitlements.update,
        )
        self.list = async_to_raw_response_wrapper(
            credit_entitlements.list,
        )
        self.delete = async_to_raw_response_wrapper(
            credit_entitlements.delete,
        )
        self.undelete = async_to_raw_response_wrapper(
            credit_entitlements.undelete,
        )

    @cached_property
    def balances(self) -> AsyncBalancesResourceWithRawResponse:
        return AsyncBalancesResourceWithRawResponse(self._credit_entitlements.balances)


class CreditEntitlementsResourceWithStreamingResponse:
    def __init__(self, credit_entitlements: CreditEntitlementsResource) -> None:
        self._credit_entitlements = credit_entitlements

        self.create = to_streamed_response_wrapper(
            credit_entitlements.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            credit_entitlements.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            credit_entitlements.update,
        )
        self.list = to_streamed_response_wrapper(
            credit_entitlements.list,
        )
        self.delete = to_streamed_response_wrapper(
            credit_entitlements.delete,
        )
        self.undelete = to_streamed_response_wrapper(
            credit_entitlements.undelete,
        )

    @cached_property
    def balances(self) -> BalancesResourceWithStreamingResponse:
        return BalancesResourceWithStreamingResponse(self._credit_entitlements.balances)


class AsyncCreditEntitlementsResourceWithStreamingResponse:
    def __init__(self, credit_entitlements: AsyncCreditEntitlementsResource) -> None:
        self._credit_entitlements = credit_entitlements

        self.create = async_to_streamed_response_wrapper(
            credit_entitlements.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            credit_entitlements.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            credit_entitlements.update,
        )
        self.list = async_to_streamed_response_wrapper(
            credit_entitlements.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            credit_entitlements.delete,
        )
        self.undelete = async_to_streamed_response_wrapper(
            credit_entitlements.undelete,
        )

    @cached_property
    def balances(self) -> AsyncBalancesResourceWithStreamingResponse:
        return AsyncBalancesResourceWithStreamingResponse(self._credit_entitlements.balances)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["Conjunction"]

Conjunction: TypeAlias = Literal["and", "or"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

from .attach_addon_param import AttachAddonParam

__all__ = ["ProductItemReqParam"]


class ProductItemReqParam(TypedDict, total=False):
    product_id: Required[str]
    """unique id of the product"""

    quantity: Required[int]

    addons: Optional[Iterable[AttachAddonParam]]
    """only valid if product is a subscription"""

    amount: Optional[int]
    """Amount the customer pays if pay_what_you_want is enabled.

    If disabled then amount will be ignored Represented in the lowest denomination
    of the currency (e.g., cents for USD). For example, to charge $1.00, pass `100`.
    Only applicable for one time payments

    If amount is not set for pay_what_you_want product, customer is allowed to
    select the amount.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .payments import (
    PaymentsResource,
    AsyncPaymentsResource,
    PaymentsResourceWithRawResponse,
    AsyncPaymentsResourceWithRawResponse,
    PaymentsResourceWithStreamingResponse,
    AsyncPaymentsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["InvoicesResource", "AsyncInvoicesResource"]


class InvoicesResource(SyncAPIResource):
    @cached_property
    def payments(self) -> PaymentsResource:
        return PaymentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> InvoicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return InvoicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InvoicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return InvoicesResourceWithStreamingResponse(self)


class AsyncInvoicesResource(AsyncAPIResource):
    @cached_property
    def payments(self) -> AsyncPaymentsResource:
        return AsyncPaymentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncInvoicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInvoicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInvoicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncInvoicesResourceWithStreamingResponse(self)


class InvoicesResourceWithRawResponse:
    def __init__(self, invoices: InvoicesResource) -> None:
        self._invoices = invoices

    @cached_property
    def payments(self) -> PaymentsResourceWithRawResponse:
        return PaymentsResourceWithRawResponse(self._invoices.payments)


class AsyncInvoicesResourceWithRawResponse:
    def __init__(self, invoices: AsyncInvoicesResource) -> None:
        self._invoices = invoices

    @cached_property
    def payments(self) -> AsyncPaymentsResourceWithRawResponse:
        return AsyncPaymentsResourceWithRawResponse(self._invoices.payments)


class InvoicesResourceWithStreamingResponse:
    def __init__(self, invoices: InvoicesResource) -> None:
        self._invoices = invoices

    @cached_property
    def payments(self) -> PaymentsResourceWithStreamingResponse:
        return PaymentsResourceWithStreamingResponse(self._invoices.payments)


class AsyncInvoicesResourceWithStreamingResponse:
    def __init__(self, invoices: AsyncInvoicesResource) -> None:
        self._invoices = invoices

    @cached_property
    def payments(self) -> AsyncPaymentsResourceWithStreamingResponse:
        return AsyncPaymentsResourceWithStreamingResponse(self._invoices.payments)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo
from .ledger_entry_type import LedgerEntryType

__all__ = ["BalanceCreateLedgerEntryParams"]


class BalanceCreateLedgerEntryParams(TypedDict, total=False):
    credit_entitlement_id: Required[str]

    amount: Required[str]
    """Amount to credit or debit"""

    entry_type: Required[LedgerEntryType]
    """Entry type: credit or debit"""

    expires_at: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Expiration for credited amount (only for credit type)"""

    idempotency_key: Optional[str]
    """Idempotency key to prevent duplicate entries"""

    metadata: Optional[Dict[str, str]]
    """
    Optional metadata (max 50 key-value pairs, key max 40 chars, value max 500
    chars)
    """

    reason: Optional[str]
    """Human-readable reason for the entry"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .country_code import CountryCode

__all__ = ["BillingAddressParam"]


class BillingAddressParam(TypedDict, total=False):
    country: Required[CountryCode]
    """Two-letter ISO country code (ISO 3166-1 alpha-2)"""

    city: Optional[str]
    """City name"""

    state: Optional[str]
    """State or province name"""

    street: Optional[str]
    """Street address including house number and unit/apartment if applicable"""

    zipcode: Optional[str]
    """Postal code or ZIP code"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .currency import Currency
from .time_interval import TimeInterval
from .cbb_overage_behavior import CbbOverageBehavior
from .cbb_proration_behavior import CbbProrationBehavior

__all__ = ["CreditEntitlementMappingResponse"]


class CreditEntitlementMappingResponse(BaseModel):
    """Response struct for credit entitlement mapping"""

    id: str
    """Unique ID of this mapping"""

    credit_entitlement_id: str
    """ID of the credit entitlement"""

    credit_entitlement_name: str
    """Name of the credit entitlement"""

    credit_entitlement_unit: str
    """Unit label for the credit entitlement"""

    credits_amount: str
    """Number of credits granted"""

    overage_behavior: CbbOverageBehavior
    """Controls how overage is handled at billing cycle end."""

    overage_enabled: bool
    """Whether overage is enabled"""

    proration_behavior: CbbProrationBehavior
    """Proration behavior for credit grants during plan changes"""

    rollover_enabled: bool
    """Whether rollover is enabled"""

    trial_credits_expire_after_trial: bool
    """Whether trial credits expire after trial"""

    currency: Optional[Currency] = None
    """Currency"""

    expires_after_days: Optional[int] = None
    """Days until credits expire"""

    low_balance_threshold_percent: Optional[int] = None
    """Low balance threshold percentage"""

    max_rollover_count: Optional[int] = None
    """Maximum rollover cycles"""

    overage_limit: Optional[str] = None
    """Overage limit"""

    price_per_unit: Optional[str] = None
    """Price per unit"""

    rollover_percentage: Optional[int] = None
    """Rollover percentage"""

    rollover_timeframe_count: Optional[int] = None
    """Rollover timeframe count"""

    rollover_timeframe_interval: Optional[TimeInterval] = None
    """Rollover timeframe interval"""

    trial_credits: Optional[str] = None
    """Trial credits"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .currency import Currency
from .time_interval import TimeInterval
from .cbb_overage_behavior import CbbOverageBehavior
from .cbb_proration_behavior import CbbProrationBehavior

__all__ = ["AttachCreditEntitlementParam"]


class AttachCreditEntitlementParam(TypedDict, total=False):
    """Request struct for attaching a credit entitlement to a product"""

    credit_entitlement_id: Required[str]
    """ID of the credit entitlement to attach"""

    credits_amount: Required[str]
    """Number of credits to grant when this product is purchased"""

    currency: Optional[Currency]
    """Currency for credit-related pricing"""

    expires_after_days: Optional[int]
    """Number of days after which credits expire"""

    low_balance_threshold_percent: Optional[int]
    """Balance threshold percentage for low balance notifications (0-100)"""

    max_rollover_count: Optional[int]
    """Maximum number of rollover cycles allowed"""

    overage_behavior: Optional[CbbOverageBehavior]
    """Controls how overage is handled at billing cycle end."""

    overage_enabled: Optional[bool]
    """Whether overage usage is allowed beyond credit balance"""

    overage_limit: Optional[str]
    """Maximum amount of overage allowed"""

    price_per_unit: Optional[str]
    """Price per credit unit for purchasing additional credits"""

    proration_behavior: Optional[CbbProrationBehavior]
    """Proration behavior for credit grants during plan changes"""

    rollover_enabled: Optional[bool]
    """Whether unused credits can roll over to the next billing period"""

    rollover_percentage: Optional[int]
    """Percentage of unused credits that can roll over (0-100)"""

    rollover_timeframe_count: Optional[int]
    """Number of timeframe units for rollover window"""

    rollover_timeframe_interval: Optional[TimeInterval]
    """Time interval for rollover window (day, week, month, year)"""

    trial_credits: Optional[str]
    """Credits granted during trial period"""

    trial_credits_expire_after_trial: Optional[bool]
    """Whether trial credits expire when trial ends"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

from .attach_addon_param import AttachAddonParam

__all__ = ["SubscriptionChangePlanParams"]


class SubscriptionChangePlanParams(TypedDict, total=False):
    product_id: Required[str]
    """Unique identifier of the product to subscribe to"""

    proration_billing_mode: Required[Literal["prorated_immediately", "full_immediately", "difference_immediately"]]
    """Proration Billing Mode"""

    quantity: Required[int]
    """Number of units to subscribe for. Must be at least 1."""

    addons: Optional[Iterable[AttachAddonParam]]
    """
    Addons for the new plan. Note : Leaving this empty would remove any existing
    addons
    """

    discount_code: Optional[str]
    """
    Optional discount code to apply to the new plan. If provided, validates and
    applies the discount to the plan change. If not provided and the subscription
    has an existing discount with `preserve_on_plan_change=true`, the existing
    discount will be preserved (if applicable to the new product).
    """

    metadata: Optional[Dict[str, str]]
    """Metadata for the payment.

    If not passed, the metadata of the subscription will be taken
    """

    on_payment_failure: Optional[Literal["prevent_change", "apply_change"]]
    """Controls behavior when the plan change payment fails.

    - `prevent_change`: Keep subscription on current plan until payment succeeds
    - `apply_change` (default): Apply plan change immediately regardless of payment
      outcome

    If not specified, uses the business-level default setting.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["WebhookListParams"]


class WebhookListParams(TypedDict, total=False):
    iterator: Optional[str]
    """The iterator returned from a prior invocation"""

    limit: Optional[int]
    """Limit the number of returned items"""
